@@ -60,7 +60,7 @@ public class LDAPAuthenticationProviderModule extends AbstractModule {
             throws GuacamoleException {
 
         // Get local environment
-        this.environment = new LocalEnvironment();
+        this.environment = LocalEnvironment.getInstance();
 
         // Store associated auth provider
         this.authProvider = authProvider;
