@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2015 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 var Guacamole = Guacamole || {};
@@ -49,10 +46,10 @@ Guacamole.AudioPlayer = function AudioPlayer() {
  * implementation of Guacamole.AudioPlayer, and thus will be properly handled
  * by Guacamole.AudioPlayer.getInstance().
  *
- * @param {String} mimetype
+ * @param {!string} mimetype
  *     The mimetype to check.
  *
- * @returns {Boolean}
+ * @returns {!boolean}
  *     true if the given mimetype is supported by any built-in
  *     Guacamole.AudioPlayer, false otherwise.
  */
@@ -71,7 +68,7 @@ Guacamole.AudioPlayer.isSupportedType = function isSupportedType(mimetype) {
  * additional parameters. Something like "audio/L8;rate=44100" would be valid,
  * however (see https://tools.ietf.org/html/rfc4856).
  *
- * @returns {String[]}
+ * @returns {!string[]}
  *     A list of all mimetypes supported by any built-in Guacamole.AudioPlayer,
  *     excluding any parameters.
  */
@@ -86,10 +83,10 @@ Guacamole.AudioPlayer.getSupportedTypes = function getSupportedTypes() {
  * audio format. If support for the given audio format is not available, null
  * is returned.
  *
- * @param {Guacamole.InputStream} stream
+ * @param {!Guacamole.InputStream} stream
  *     The Guacamole.InputStream to read audio data from.
  *
- * @param {String} mimetype
+ * @param {!string} mimetype
  *     The mimetype of the audio data in the provided stream.
  *
  * @return {Guacamole.AudioPlayer}
@@ -115,10 +112,10 @@ Guacamole.AudioPlayer.getInstance = function getInstance(stream, mimetype) {
  *
  * @constructor
  * @augments Guacamole.AudioPlayer
- * @param {Guacamole.InputStream} stream
+ * @param {!Guacamole.InputStream} stream
  *     The Guacamole.InputStream to read audio data from.
  *
- * @param {String} mimetype
+ * @param {!string} mimetype
  *     The mimetype of the audio data in the provided stream, which must be a
  *     "audio/L8" or "audio/L16" mimetype with necessary parameters, such as:
  *     "audio/L16;rate=44100,channels=2".
@@ -129,9 +126,9 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * The format of audio this player will decode.
      *
      * @private
-     * @type {Guacamole.RawAudioPlayer._Format}
+     * @type {Guacamole.RawAudioFormat}
      */
-    var format = Guacamole.RawAudioPlayer._Format.parse(mimetype);
+    var format = Guacamole.RawAudioFormat.parse(mimetype);
 
     /**
      * An instance of a Web Audio API AudioContext object, or null if the
@@ -140,25 +137,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * @private
      * @type {AudioContext}
      */
-    var context = (function getAudioContext() {
-
-        // Fallback to Webkit-specific AudioContext implementation
-        var AudioContext = window.AudioContext || window.webkitAudioContext;
-
-        // Get new AudioContext instance if Web Audio API is supported
-        if (AudioContext) {
-            try {
-                return new AudioContext();
-            }
-            catch (e) {
-                // Do not use Web Audio API if not allowed by browser
-            }
-        }
-
-        // Web Audio API not supported
-        return null;
-
-    })();
+    var context = Guacamole.AudioContextFactory.getAudioContext();
 
     /**
      * The earliest possible time that the next packet could play without
@@ -167,7 +146,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * resolution.
      *
      * @private
-     * @type {Number}
+     * @type {!number}
      */
     var nextPacketTime = context.currentTime;
 
@@ -176,7 +155,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * provided with this Guacamole.RawAudioPlayer was created.
      *
      * @private
-     * @type {Guacamole.ArrayBufferReader}
+     * @type {!Guacamole.ArrayBufferReader}
      */
     var reader = new Guacamole.ArrayBufferReader(stream);
 
@@ -188,7 +167,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      *
      * @private
      * @constant
-     * @type {Number}
+     * @type {!number}
      */
     var MIN_SPLIT_SIZE = 0.02;
 
@@ -198,7 +177,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * roughly one third of a second.
      *
      * @private
-     * @type {Number}
+     * @type {!number}
      */
     var maxLatency = 0.3;
 
@@ -218,7 +197,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * sample, and will be 128 for 8-bit audio and 32768 for 16-bit audio.
      *
      * @private
-     * @type {Number}
+     * @type {!number}
      */
     var maxSampleValue = (format.bytesPerSample === 1) ? 128 : 32768;
 
@@ -230,7 +209,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * no further modifications can be made to that packet.
      *
      * @private
-     * @type {SampleArray[]}
+     * @type {!SampleArray[]}
      */
     var packetQueue = [];
 
@@ -239,7 +218,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * containing the concatenation of those packets.
      *
      * @private
-     * @param {SampleArray[]} packets
+     * @param {!SampleArray[]} packets
      *     The array of audio packets to concatenate.
      *
      * @returns {SampleArray}
@@ -279,10 +258,10 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * the originally-provided audio packet is returned.
      *
      * @private
-     * @param {SampleArray} data
+     * @param {!SampleArray} data
      *     The audio packet to split.
      *
-     * @returns {SampleArray[]}
+     * @returns {!SampleArray[]}
      *     An array of audio packets containing the result of splitting the
      *     provided audio packet. If splitting is possible, this array will
      *     contain two packets. If splitting is not possible, this array will
@@ -347,7 +326,7 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * conversions will be performed automatically internally.
      *
      * @private
-     * @param {ArrayBuffer} data
+     * @param {!ArrayBuffer} data
      *     A raw packet of audio data that should be pushed onto the audio
      *     playback queue.
      */
@@ -364,7 +343,8 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * @private
      * @returns {SampleArray}
      *     A packet of audio data pulled from the beginning of the playback
-     *     queue.
+     *     queue. If there is no audio currently in the playback queue, this
+     *     will be null.
      */
     var shiftAudioPacket = function shiftAudioPacket() {
 
@@ -388,11 +368,11 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
      * into isolated planes of channel-specific data.
      *
      * @private
-     * @param {SampleArray} data
+     * @param {!SampleArray} data
      *     The raw audio packet that should be converted into a Web Audio API
      *     AudioBuffer.
      *
-     * @returns {AudioBuffer}
+     * @returns {!AudioBuffer}
      *     A new Web Audio API AudioBuffer containing the provided audio data,
      *     converted to the format used by the Web Audio API.
      */
@@ -477,151 +457,24 @@ Guacamole.RawAudioPlayer = function RawAudioPlayer(stream, mimetype) {
 
 Guacamole.RawAudioPlayer.prototype = new Guacamole.AudioPlayer();
 
-/**
- * A description of the format of raw PCM audio received by a
- * Guacamole.RawAudioPlayer. This object describes the number of bytes per
- * sample, the number of channels, and the overall sample rate.
- *
- * @private
- * @constructor
- * @param {Guacamole.RawAudioPlayer._Format|Object} template
- *     The object whose properties should be copied into the corresponding
- *     properties of the new Guacamole.RawAudioPlayer._Format.
- */
-Guacamole.RawAudioPlayer._Format = function _Format(template) {
-
-    /**
-     * The number of bytes in each sample of audio data. This value is
-     * independent of the number of channels.
-     *
-     * @type {Number}
-     */
-    this.bytesPerSample = template.bytesPerSample;
-
-    /**
-     * The number of audio channels (ie: 1 for mono, 2 for stereo).
-     *
-     * @type {Number}
-     */
-    this.channels = template.channels;
-
-    /**
-     * The number of samples per second, per channel.
-     *
-     * @type {Number}
-     */
-    this.rate = template.rate;
-
-};
-
-/**
- * Parses the given mimetype, returning a new Guacamole.RawAudioPlayer._Format
- * which describes the type of raw audio data represented by that mimetype. If
- * the mimetype is not supported by Guacamole.RawAudioPlayer, null is returned.
- *
- * @private
- * @param {String} mimetype
- *     The audio mimetype to parse.
- *
- * @returns {Guacamole.RawAudioPlayer._Format}
- *     A new Guacamole.RawAudioPlayer._Format which describes the type of raw
- *     audio data represented by the given mimetype, or null if the given
- *     mimetype is not supported.
- */
-Guacamole.RawAudioPlayer._Format.parse = function parseFormat(mimetype) {
-
-    var bytesPerSample;
-
-    // Rate is absolutely required - if null is still present later, the
-    // mimetype must not be supported
-    var rate = null;
-
-    // Default for both "audio/L8" and "audio/L16" is one channel
-    var channels = 1;
-
-    // "audio/L8" has one byte per sample
-    if (mimetype.substring(0, 9) === 'audio/L8;') {
-        mimetype = mimetype.substring(9);
-        bytesPerSample = 1;
-    }
-
-    // "audio/L16" has two bytes per sample
-    else if (mimetype.substring(0, 10) === 'audio/L16;') {
-        mimetype = mimetype.substring(10);
-        bytesPerSample = 2;
-    }
-
-    // All other types are unsupported
-    else
-        return null;
-
-    // Parse all parameters
-    var parameters = mimetype.split(',');
-    for (var i = 0; i < parameters.length; i++) {
-
-        var parameter = parameters[i];
-
-        // All parameters must have an equals sign separating name from value
-        var equals = parameter.indexOf('=');
-        if (equals === -1)
-            return null;
-
-        // Parse name and value from parameter string
-        var name  = parameter.substring(0, equals);
-        var value = parameter.substring(equals+1);
-
-        // Handle each supported parameter
-        switch (name) {
-
-            // Number of audio channels
-            case 'channels':
-                channels = parseInt(value);
-                break;
-
-            // Sample rate
-            case 'rate':
-                rate = parseInt(value);
-                break;
-
-            // All other parameters are unsupported
-            default:
-                return null;
-
-        }
-
-    };
-
-    // The rate parameter is required
-    if (rate === null)
-        return null;
-
-    // Return parsed format details
-    return new Guacamole.RawAudioPlayer._Format({
-        bytesPerSample : bytesPerSample,
-        channels       : channels,
-        rate           : rate
-    });
-
-};
-
 /**
  * Determines whether the given mimetype is supported by
  * Guacamole.RawAudioPlayer.
  *
- * @param {String} mimetype
+ * @param {!string} mimetype
  *     The mimetype to check.
  *
- * @returns {Boolean}
+ * @returns {!boolean}
  *     true if the given mimetype is supported by Guacamole.RawAudioPlayer,
  *     false otherwise.
  */
 Guacamole.RawAudioPlayer.isSupportedType = function isSupportedType(mimetype) {
 
     // No supported types if no Web Audio API
-    if (!window.AudioContext && !window.webkitAudioContext)
+    if (!Guacamole.AudioContextFactory.getAudioContext())
         return false;
 
-    return Guacamole.RawAudioPlayer._Format.parse(mimetype) !== null;
+    return Guacamole.RawAudioFormat.parse(mimetype) !== null;
 
 };
 
@@ -633,7 +486,7 @@ Guacamole.RawAudioPlayer.isSupportedType = function isSupportedType(mimetype) {
  * additional parameters. Something like "audio/L8;rate=44100" would be valid,
  * however (see https://tools.ietf.org/html/rfc4856).
  *
- * @returns {String[]}
+ * @returns {!string[]}
  *     A list of all mimetypes supported by Guacamole.RawAudioPlayer, excluding
  *     any parameters. If the necessary JavaScript APIs for playing raw audio
  *     are absent, this list will be empty.
@@ -641,7 +494,7 @@ Guacamole.RawAudioPlayer.isSupportedType = function isSupportedType(mimetype) {
 Guacamole.RawAudioPlayer.getSupportedTypes = function getSupportedTypes() {
 
     // No supported types if no Web Audio API
-    if (!window.AudioContext && !window.webkitAudioContext)
+    if (!Guacamole.AudioContextFactory.getAudioContext())
         return [];
 
     // We support 8-bit and 16-bit raw PCM
