@@ -38,15 +38,9 @@
 ##     subdirectories within this directory, and files will thus be grouped by
 ##     extension type.
 ##
-## @param BUILD_PROFILE
-##     The build profile that will be passed to Maven build process. Defaults
-##     to empty string. Can be set to "lgpl-extensions" to e.g. include
-##     RADIUS authentication extension.
-##
 
 BUILD_DIR="$1"
 DESTINATION="$2"
-BUILD_PROFILE="$3"
 
 #
 # Create destination, if it does not yet exist
@@ -60,16 +54,11 @@ mkdir -p "$DESTINATION"
 
 cd "$BUILD_DIR"
 
-# Required for build leveraging PhantomJS for unit testing (without this, the
-# build fails with "libssl_conf.so: cannot open shared object file: No such
-# file or directory")
-export OPENSSL_CONF=/etc/ssl
+#
+# Run the maven build, applying any arbitrary provided maven arguments.
+#
 
-if [ -z "$BUILD_PROFILE" ]; then
-    mvn package
-else
-    mvn -P "$BUILD_PROFILE" package
-fi
+mvn $MAVEN_ARGUMENTS package
 
 #
 # Copy guacamole.war to destination
@@ -94,7 +83,7 @@ tar -xzf extensions/guacamole-auth-jdbc/modules/guacamole-auth-jdbc-dist/target/
 #
 
 echo "Downloading MySQL Connector/J ..."
-curl -L "https://dev.mysql.com/get/Downloads/Connector-J/mysql-connector-java-5.1.46.tar.gz" | \
+curl -L "https://dev.mysql.com/get/Downloads/Connector-J/mysql-connector-j-$MYSQL_JDBC_VERSION.tar.gz" | \
 tar -xz                        \
     -C "$DESTINATION/mysql/"   \
     --wildcards                \
@@ -108,7 +97,8 @@ tar -xz                        \
 #
 
 echo "Downloading PostgreSQL JDBC driver ..."
-curl -L "https://jdbc.postgresql.org/download/postgresql-42.2.24.jre7.jar" > "$DESTINATION/postgresql/postgresql-42.2.24.jre7.jar"
+curl -L "https://jdbc.postgresql.org/download/postgresql-$PGSQL_JDBC_VERSION.jar" \
+    > "$DESTINATION/postgresql/postgresql-$PGSQL_JDBC_VERSION.jar"
 
 #
 # Copy SSO auth extensions
@@ -126,14 +116,8 @@ tar -xzf extensions/guacamole-auth-sso/modules/guacamole-auth-sso-dist/target/*.
 #
 
 echo "Downloading SQL Server JDBC driver ..."
-curl -L "https://go.microsoft.com/fwlink/?linkid=2183223&clcid=0x409" | \
-tar -xz                        \
-    -C "$DESTINATION/sqlserver/"   \
-    --wildcards                \
-    --no-anchored              \
-    --no-wildcards-match-slash \
-    --strip-components=2       \
-    "mssql-jdbc-*.jre8.jar"
+curl -L "https://github.com/microsoft/mssql-jdbc/releases/download/v$MSSQL_JDBC_VERSION/mssql-jdbc-$MSSQL_JDBC_VERSION.jre8.jar" \
+    > "$DESTINATION/sqlserver/mssql-jdbc-$MSSQL_JDBC_VERSION.jre8.jar"   \
 
 #
 # Copy LDAP auth extension and schema modifications
@@ -198,3 +182,12 @@ if [ -f extensions/guacamole-auth-json/target/guacamole-auth-json*.jar ]; then
     mkdir -p "$DESTINATION/json"
     cp extensions/guacamole-auth-json/target/guacamole-auth-json*.jar "$DESTINATION/json"
 fi
+
+#
+# Copy history recording storage extension if it was built
+#
+
+if [ -f extensions/guacamole-history-recording-storage/target/guacamole-history-recording-storage*.jar ]; then
+    mkdir -p "$DESTINATION/recordings"
+    cp extensions/guacamole-history-recording-storage/target/guacamole-history-recording-storage*.jar "$DESTINATION/recordings"
+fi
